package com.idea.buildManage.excel;

import cn.hutool.core.util.IdUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSONObject;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkInfoService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

public class ParkFloorDiscListener extends AnalysisEventListener<ParkFloorDiscExcel> {

    private List<ParkFloorDisc> list = new ArrayList<>();

    private List<ParkFloorDiscExcel> excelList = new ArrayList<>();

    private List<ParkFloorDiscExcel> badList = new ArrayList<>();

    Integer count;

    private JSONObject jsonObject;

    private ParkFloorDiscService floorDiscService;
    private ParkInfoService parkInfoService;
    private SysDictService sysDictService;


    public ParkFloorDiscListener(ParkFloorDiscService floorDiscService,ParkInfoService parkInfoService,
                                 SysDictService sysDictService, JSONObject jsonObject) {
        this.floorDiscService = floorDiscService;
        this.parkInfoService = parkInfoService;
        this.sysDictService = sysDictService;
        this.jsonObject = jsonObject;
    }


    @Override
    public void invoke(ParkFloorDiscExcel parkFloorDiscExcel, AnalysisContext analysisContext) {
        excelList.add(parkFloorDiscExcel);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {


        // 支付方式字典
        List<Map<String, String>> paymentMethods = sysDictService.selectDictList("PAYMENT_METHODS");

        if(CollectionUtils.isNotEmpty(excelList)){
            ParkFloorDisc data;
            ParkFloorDiscExcel excel = null;
            Map<String, String> discNameIdMap = floorDiscService.getNameIdMap();
            Map<String, String> parkNameIdMap = parkInfoService.getNameIdMap();

            for(int i = 0; i < excelList.size(); i++){
                try {
                    data = new ParkFloorDisc();
                    excel = excelList.get(i);
                    if(StringUtils.isEmpty(excel.getGroupName())){
                        throw new RuntimeException("楼盘/小区名称为空");
                    }
                    if(StringUtils.isEmpty(excel.getDiscName())){
                        throw new RuntimeException("楼分期名称为空");
                    }
                    data.setName(excel.getDiscName());
                    // 判断小区是否存在
                    if(StringUtils.isEmpty(parkNameIdMap.get(excel.getGroupName()))){
                        throw new RuntimeException("楼盘/小区不存在");
                    }
                    data.setGroupId(parkNameIdMap.get(excel.getGroupName()));

                    // 房屋买卖单价
                    if (StringUtils.isNotEmpty(excel.getUnitPrice())) {
                        try {
                            data.setUnitPrice(new BigDecimal(excel.getUnitPrice()));
                        } catch (NumberFormatException e) {
                            throw new RuntimeException("房屋买卖单价（元/㎡）格式有误");
                        }
                    }

                    // 物业费单价
                    if (StringUtils.isNotEmpty(excel.getPropertyUnitPrice())) {
                        try {
                            data.setPropertyUnitPrice(new BigDecimal(excel.getPropertyUnitPrice()));
                        } catch (NumberFormatException e) {
                            throw new RuntimeException("物业费单价（元/㎡）格式有误");
                        }
                    }

                    // 住宅专项维修资金
                    if (StringUtils.isNotEmpty(excel.getMaintenanceFunds())) {
                        try {
                            data.setMaintenanceFunds(new BigDecimal(excel.getMaintenanceFunds()));
                        } catch (NumberFormatException e) {
                            throw new RuntimeException("住宅专项维修资金（元/㎡）格式有误");
                        }
                    }

                    // 认购金
                    if (StringUtils.isNotEmpty(excel.getSubscriptionFunds())) {
                        try {
                            data.setSubscriptionFunds(new BigDecimal(excel.getSubscriptionFunds()));
                        } catch (NumberFormatException e) {
                            throw new RuntimeException("认购金（元）格式有误");
                        }
                    }

                    // 共持比例
                    if (StringUtils.isNotEmpty(excel.getProportion())) {
                        try {
                            data.setProportion(Double.valueOf(excel.getProportion()));
                        } catch (NumberFormatException e) {
                            throw new RuntimeException("共持比例格式有误");
                        }
                    }

                    // 支付方式
                    if (StringUtils.isNotEmpty(excel.getPaymentMethods())) {
                        for (Map<String, String> stringStringMap : paymentMethods) {
                            if (stringStringMap.get("label").equals(excel.getPaymentMethods())) {
                                data.setPaymentMethods(Integer.valueOf(stringStringMap.get("value")));
                            }
                        }
                        // 没匹配上的情况
                        if(data.getPaymentMethods() == null){
                            throw new RuntimeException("支付方式格式有误");
                        }
                    }
                    data.setId(IdUtil.simpleUUID());
                    data.setCreatedId(ShiroUtils.getUserId());
                    data.setCreatedAt(new Date());
                    list.add(data);

                }catch (Exception e){
                    e.printStackTrace();
                    String message = e.getMessage();
                    excel.setReason(message);
                    badList.add(excel);
                }
            }

            count = list.size();
            jsonObject.put("successCount", count);
            jsonObject.put("errorList", badList);
            jsonObject.put("errorCount", badList.size());
            //保存
            saveData();

            excelList.clear();
            list.clear();

        }else {
            throw new RuntimeException("excel无数据");
        }


    }

    private void saveData(){
        if (CollectionUtils.isNotEmpty(list)) {
            floorDiscService.insertBatch(list);
        }
    }
}
