package com.idea.buildManage.service;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.dto.MnpBuildingDto;
import com.idea.buildManage.mapper.MnpBuildingExtendMapper;
import com.idea.buildManage.mapper.MnpBuildingMapper;
import com.idea.buildManage.model.*;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.mapper.FrameDataMapper;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class MnpBuildingService implements BaseService<MnpBuilding, MnpBuildingExample> {

    //文件mapper
    @Autowired
    private MnpBuildingMapper modelMapper;
    @Autowired
    private FrameDataMapper frameDataMapper;
    @Autowired
    private ParkRoomService parkRoomService;
    @Autowired
    private MnpBuildingExtendMapper extendMapper;
    
    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<MnpBuildingDto> list(Tablepar tablepar, MnpBuildingDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<MnpBuildingDto> list = extendMapper.listByModel(model);
        PageInfo<MnpBuildingDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询指定
     *
     * @return
     */
    public List<MnpBuilding> listAll(MnpBuilding model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    private MnpBuildingExample getCondition(MnpBuilding model, DateTrans dt) {
        MnpBuildingExample ex = new MnpBuildingExample();
        return ex;
    }

    @Override
    public int deleteByPrimaryKey(String id) {
        // 先查询该楼栋下是否存在房间
        ParkRoomExample parkRoomExample = new ParkRoomExample();
        parkRoomExample.createCriteria().andBuildIdEqualTo(id);
        long count = parkRoomService.countByExample(parkRoomExample);
        if (count > 0) {
            throw new RuntimeException("该楼栋下存在房间,无法删除!");
        }
        return modelMapper.deleteByPrimaryKey(id);
    }

    @Override
    public MnpBuilding selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }

    @Override
    public int updateByPrimaryKeySelective(MnpBuilding record) {
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(MnpBuilding record, MnpBuildingExample example) {
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(MnpBuilding record, MnpBuildingExample example) {
        return modelMapper.updateByExample(record, example);
    }

    public int updateByPrimaryKey(MnpBuilding record) {
        return modelMapper.updateByPrimaryKey(record);
    }

    @Override
    public List<MnpBuilding> selectByExample(MnpBuildingExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(MnpBuildingExample example) {
        return modelMapper.countByExample(example);
    }

    public long countByExample(MnpBuilding model, DateTrans dt) {
        return modelMapper.countByExample(getCondition(model, dt));
    }

    @Override
    public int deleteByExample(MnpBuildingExample example) {
        return modelMapper.deleteByExample(example);
    }

    public int insert(MnpBuilding record) {
        record.setId(IdUtil.simpleUUID());
        return insertWithoutId(record);
    }

    public int insertWithoutId(MnpBuilding record) {
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(MnpBuilding record) {
        record.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(record);
    }


    public List<MnpBuildingDto> excelListAll(MnpBuildingDto model) {

        List<MnpBuildingDto> list = extendMapper.listByModel(model);
        return list;
    }

    /**
     * 根据合同id查询楼栋
     * @param contractId
     * @return
     */
    public List<MnpBuilding> selectByContractId(String contractId){
       return extendMapper.selectByContractId(contractId);
    }

    /**
     * 批量插入
     * @param list
     */
    public void insertBatch(List<MnpBuilding> list){
        extendMapper.batchInsert(list);
    }

    public Map<String,String> getNameIdMap() {
        Map<String, String> map = new HashMap<>();
        List<MnpBuilding> list = selectByExample(new MnpBuildingExample());
        if (CollectionUtils.isNotEmpty(list)) {
            map = list.stream().collect(Collectors.toMap(MnpBuilding::getBuildNum, MnpBuilding::getId, (v1, v2) ->v1));
        }
        return map;
    }

    /**
     * 楼栋删除
     * @param id
     * @return
     */
    public int delete(String id){
        ParkRoomExample roomExample = new ParkRoomExample();
        roomExample.createCriteria().andBuildIdEqualTo(id);
        List<ParkRoom> parkRooms = parkRoomService.selectByExample(roomExample);
        if(CollectionUtils.isNotEmpty(parkRooms)){
            throw new RuntimeException("该数据有子房间存在,无法删除");
        }
        return deleteByPrimaryKey(id);
    }

    public String getOrderNum(MnpBuilding model){
        MnpBuildingExample example = new MnpBuildingExample();
        example.createCriteria().andDiscIdEqualTo(model.getDiscId());
        Long count = countByExample(example);
        String number = "";
        if(count.doubleValue() == 0){
            number = "001";
        }else {
            count++;
            number = String.format("%03d", count);
        }
        return number;
    }


}
