package com.idea.customerManagement.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkFloorDiscMapper;
import com.idea.buildManage.mapper.ParkInfoMapper;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.dto.IntentionalDepositDto;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.IntentionalDepositExtendMapper;
import com.idea.customerManagement.mapper.IntentionalDepositMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.*;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.model.PayLog;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Service
public class IntentionalDepositService implements BaseService<IntentionalDeposit, IntentionalDepositExample> {

    @Autowired
    private IntentionalDepositMapper modelMapper;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private ParkInfoMapper parkInfoMapper;
    @Autowired
    private ParkFloorDiscMapper parkFloorDiscMapper;
    @Autowired
    private FrameUserService frameUserService;
    @Autowired
    private IntentionalDepositExtendMapper extendMapper;
    @Autowired
    private ParkRoomExtendMapper roomExtendMapper;
    @Autowired
    private RoomSelectionInfoMapper roomSelectionInfoMapper;
    @Autowired
    private ParkRoomMapper parkRoomMapper;
    @Autowired
    private ContractManageService contractManageService;
    @Autowired
    private PayLogMapper payLogMapper;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(IntentionalDeposit intentionalDeposit) {
        intentionalDeposit.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(intentionalDeposit);
    }

    public int insertWithoutId(IntentionalDeposit intentionalDeposit) {
        return modelMapper.insertSelective(intentionalDeposit);
    }

    @Override
    public IntentionalDeposit selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(IntentionalDeposit intentionalDeposit) {
        return modelMapper.updateByPrimaryKeySelective(intentionalDeposit);
    }

    @Override
    public int updateByExampleSelective(IntentionalDeposit intentionalDeposit, IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.updateByExampleSelective(intentionalDeposit,intentionalDepositExample);
    }

    @Override
    public int updateByExample(IntentionalDeposit intentionalDeposit, IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.updateByExample(intentionalDeposit,intentionalDepositExample);
    }

    @Override
    public List<IntentionalDeposit> selectByExample(IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.selectByExample(intentionalDepositExample);
    }

    @Override
    public long countByExample(IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.countByExample(intentionalDepositExample);
    }

    @Override
    public int deleteByExample(IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.deleteByExample(intentionalDepositExample);
    }

    /**
     * 获取收据流水号（意向金编号）
     * @param customerManagementId
     * @return
     */
    public String getSerialNumber(String customerManagementId){
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(customerManagementId);
        String groupId = customerManagement.getGroupId();
        String discId = customerManagement.getDiscId();
        // 查找这个小区已收取意向金的 数据量
        IntentionalDepositExample example = new IntentionalDepositExample();
        example.createCriteria().andGroupIdEqualTo(groupId);
        Long count = countByExample(example);
        String number = "";
        if(count.doubleValue() == 0){
            number = "00001";
        }else {
            count ++;
            number = String.format("%05d", count);
        }
        ParkInfo parkInfo = parkInfoMapper.selectByPrimaryKey(groupId);
        ParkFloorDisc parkFloorDisc = parkFloorDiscMapper.selectByPrimaryKey(discId);
        String serialNumber = parkInfo.getGroupName()+ parkFloorDisc.getName() + "YXJ" + number;
        return serialNumber;
    }

    /**
     * 获取收据流水号（意向金编号）
     * @param id
     * @return
     */
    public String getDepositSerialNumber(String id){

        IntentionalDeposit deposit = selectByPrimaryKey(id);
        String groupId = deposit.getGroupId();
        String discId = deposit.getDiscId();
        ParkInfo parkInfo = parkInfoMapper.selectByPrimaryKey(groupId);
        ParkFloorDisc parkFloorDisc = parkFloorDiscMapper.selectByPrimaryKey(discId);
        // 查找这个小区已收取认购金的 数据量
        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andGroupIdEqualTo(groupId).andStatusEqualTo(2);
        Long count = roomSelectionInfoMapper.countByExample(example);
        String number = "";
        if(count.doubleValue() == 0){
            number = "00001";
        }else {
            count ++;
            number = String.format("%05d", count);
        }
        String serialNumber = parkInfo.getGroupName()+ parkFloorDisc.getName() + "-DJ" + number;
        return serialNumber;
    }


    /**
     * 意向金登记
     * @param model
     * @return
     */
    public int add(IntentionalDeposit model){

        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(model.getCustomerManagementId());
        // 已收取
        model.setStatus(1);
        model.setGroupId(customerManagement.getGroupId());
        model.setDiscId(customerManagement.getDiscId());
        model.setCreatedAt(new Date());
        model.setCreatedId(ShiroUtils.getUserId());
        return insertSelective(model);
    }

    /**
     * 意向金修改
     * @param model
     * @return
     */
    public int edit(IntentionalDeposit model){

        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(model.getCustomerManagementId());
        model.setGroupId(customerManagement.getGroupId());
        model.setDiscId(customerManagement.getDiscId());
        model.setUpdatedAt(new Date());
        model.setUpdatedId(ShiroUtils.getUserId());
        return updateByPrimaryKeySelective(model);
    }

    /**
     * 根据客户id查找意向金
     * @param customerManagementId
     * @return
     */
    public IntentionalDepositDto getByCustomerManagementId(String customerManagementId){

        IntentionalDepositExample example = new IntentionalDepositExample();
        example.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<IntentionalDeposit> intentionalDeposits = modelMapper.selectByExampleWithBLOBs(example);
        if(CollectionUtils.isNotEmpty(intentionalDeposits)){
            IntentionalDepositDto result = new IntentionalDepositDto();
            IntentionalDeposit intentionalDeposit = intentionalDeposits.get(0);
            BeanUtil.copyProperties(intentionalDeposit,result);
            FrameUser frameUser = frameUserService.selectByPrimaryKey(intentionalDeposit.getCreatedId());
            if(ObjectUtil.isNotNull(frameUser)){
                result.setCreatedName(frameUser.getTruename());
            }
            return result;
        }
        return null;

    }

    /**
     * 分页
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    public PageInfo<IntentionalDepositDto> listByModel(Tablepar tablepar, IntentionalDepositDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<IntentionalDepositDto> list = extendMapper.listByModel(model);
        PageInfo<IntentionalDepositDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 意向金转定金
     * @param model
     * @return
     */
    public int toDeposit(IntentionalDeposit model) {

        String houseId = model.getHouseId();
        String customerManagementId = model.getCustomerManagementId();

        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(customerManagementId);
        // 购房状态 已选房
        customerManagement.setStatus(2);
        customerManagement.setHouseId(houseId);
        StringBuilder builder = new StringBuilder();

        // 拼接房屋名称
        ParkRoomResponse response = roomExtendMapper.getFullName(houseId);
        builder.append(response.getBuildName()).append("-").append(response.getRoomNo()).append(",");
        if (builder.length() > 0) {
            builder.deleteCharAt(builder.length() - 1);
        }
        customerManagement.setHouseName(builder.toString());
        customerManagementMapper.updateByPrimaryKeySelective(customerManagement);

        // 新增选房信息
        RoomSelectionInfo roomSelectionInfo = new RoomSelectionInfo();
        roomSelectionInfo.setCustomerManagementId(customerManagementId);
        roomSelectionInfo.setHouseId(response.getId());
        roomSelectionInfo.setBuildId(response.getBuildId());
        roomSelectionInfo.setDiscId(response.getDiscId());
        roomSelectionInfo.setGroupId(response.getGroupId());
        roomSelectionInfo.setCreatedAt(new Date());
        roomSelectionInfo.setCreatedId(ShiroUtils.getUserId());
        roomSelectionInfo.setReceivableMoney(response.getSubscriptionFunds());
        // 收取状态 意向金转入
        roomSelectionInfo.setStatus(3);
        roomSelectionInfo.setId(IdUtil.simpleUUID());
        roomSelectionInfo.setDepositNumber(model.getDepositSerialNumber());
        roomSelectionInfoMapper.insertSelective(roomSelectionInfo);

        // 转定金后生产合同
        String contractId = IdUtil.simpleUUID();
        contractManageService.addContract(roomSelectionInfo,contractId);

        // 新增一条定金收款记录
        PayLog payLog = new PayLog();
        payLog.setId(IdUtil.simpleUUID());
        payLog.setHouseId(houseId);
        payLog.setContractId(contractId);
        payLog.setCustomerManagementId(customerManagementId);
        // 收款类型 预收款
        payLog.setPayType(1);
        // 款项内容 定金
        payLog.setContentType(1);
        payLog.setPayMoney(model.getReceivedAmount());
        payLog.setPayTime(new Date());
        payLog.setPaymentMethod(model.getPaymentMethod());
        payLog.setStatus(0);
        payLogMapper.insertSelective(payLog);

        // 更新房间状态
        ParkRoom parkRoom = new ParkRoom();
        parkRoom.setId(response.getId());
        parkRoom.setIsChoose(1);
        parkRoomMapper.updateByPrimaryKeySelective(parkRoom);


        // 转定金
        model.setStatus(2);
        model.setHandleId(ShiroUtils.getUserId());
        model.setHandleDate(new Date());
        modelMapper.updateByPrimaryKeySelective(model);

        return 1;

    }

    /**
     * 退意向金
     * @param model
     * @return
     */
    public int refund(IntentionalDeposit model){

        // 退意向金
        model.setStatus(3);
        model.setHandleId(ShiroUtils.getUserId());
        model.setHandleDate(new Date());
        int result = modelMapper.updateByPrimaryKeySelective(model);
        return result;
    }

    public IntentionalDepositDto getById(String id) {
        IntentionalDeposit intentionalDeposit = modelMapper.selectByPrimaryKey(id);
        IntentionalDepositDto intentionalDepositDto = new IntentionalDepositDto();
        String handleName = "";
        if(StringUtils.isNotEmpty(intentionalDeposit.getHandleId())){
            FrameUser frameUser = frameUserService.selectByPrimaryKey(intentionalDeposit.getHandleId());
            handleName = frameUser == null ? "" : frameUser.getTruename();
        }
        BeanUtil.copyProperties(intentionalDeposit,intentionalDepositDto);
        intentionalDepositDto.setHandleName(handleName);
        return intentionalDepositDto;
    }

}
