package com.idea.demo.service;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.demo.mapper.DemoStudentMapper;
import com.idea.demo.model.DemoStudent;
import com.idea.demo.model.DemoStudentExample;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.util.DateUtils;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
public class DemoStudentService implements BaseService<DemoStudent, DemoStudentExample> {

    //文件mapper
    @Autowired
    private DemoStudentMapper modelMapper;

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<DemoStudent> list(Tablepar tablepar, DemoStudent model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<DemoStudent> list = modelMapper.selectByExample(getCondition(model,dt));
        PageInfo<DemoStudent> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询指定
     *
     * @return
     */
    public List<DemoStudent> listAll(DemoStudent model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model,dt));
    }

    private DemoStudentExample getCondition(DemoStudent model, DateTrans dt) {
        DemoStudentExample ex = new DemoStudentExample();
        ex.setOrderByClause("created_at asc");
        DemoStudentExample.Criteria criteria = ex.createCriteria();

        if(StringUtils.isNotEmpty(model.getName())){
            criteria.andNameLike("%"+model.getName()+"%");
        }
        if(StringUtils.isNotEmpty(model.getGender())){
            criteria.andGenderEqualTo(model.getGender());
        }

        if(dt != null){
            if(StringUtils.isNotEmpty(dt.getDate_from_1())){
                criteria.andCreatedAtGreaterThanOrEqualTo(DateUtils.string2Date(dt.getDate_from_1()));
            }
            if(StringUtils.isNotEmpty(dt.getDate_to_1())){
                criteria.andCreatedAtLessThanOrEqualTo(DateUtils.string2Date(dt.getDate_to_1()));
            }
        }

        return ex;
    }

    @Override
    public int deleteByPrimaryKey(String ids) {
        List<String> lista = Convert.toListStrArray(ids);
        DemoStudentExample example = new DemoStudentExample();
        example.createCriteria().andIdIn(lista);
        return modelMapper.deleteByExample(example);
    }

    @Override
    public DemoStudent selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }

    @Override
    public int updateByPrimaryKeySelective(DemoStudent record) {
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(DemoStudent record, DemoStudentExample example) {
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(DemoStudent record, DemoStudentExample example) {
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<DemoStudent> selectByExample(DemoStudentExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(DemoStudentExample example) {
        return modelMapper.countByExample(example);
    }

    public long countByExample(DemoStudent model, DateTrans dt) {
        return modelMapper.countByExample(getCondition(model,dt));
    }

    @Override
    public int deleteByExample(DemoStudentExample example) {
        return modelMapper.deleteByExample(example);
    }

    public int insert(DemoStudent record){
        record.setId(SnowflakeIdWorker.getUUID());
        return insertWithoutId(record);
    }

    public int insertWithoutId(DemoStudent record){
        record.setCreatedAt(new Date());
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(DemoStudent record) {
        record.setId(SnowflakeIdWorker.getUUID());
        record.setCreatedAt(new Date());
        return modelMapper.insertSelective(record);
    }

}
