package com.idea.paymentManagement.service;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkRoom;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.mapper.ContractManageMapper;
import com.idea.customerManagement.model.ContractManage;
import com.idea.invoice.model.InvoiceBank;
import com.idea.invoice.model.InvoiceBankExample;
import com.idea.invoice.service.InvoiceBankService;
import com.idea.paymentManagement.mapper.MaintenanceFundsManagementExtendMapper;
import com.idea.paymentManagement.mapper.MaintenanceFundsManagementMapper;
import com.idea.paymentManagement.model.MaintenanceFundsManagement;
import com.idea.paymentManagement.model.MaintenanceFundsManagementExample;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Service
public class MaintenanceFundsManagementService implements BaseService<MaintenanceFundsManagement, MaintenanceFundsManagementExample> {

    @Autowired
    private MaintenanceFundsManagementMapper modelMapper;
    @Autowired
    private MaintenanceFundsManagementExtendMapper extendMapper;
    @Autowired
    private ContractManageMapper contractManageMapper;
    @Autowired
    private InvoiceBankService invoiceBankService;
    @Autowired
    private ParkRoomMapper parkRoomMapper;



    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(MaintenanceFundsManagement maintenanceFundsManagement) {
        maintenanceFundsManagement.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(maintenanceFundsManagement);
    }

    public int insertWithoutId(MaintenanceFundsManagement maintenanceFundsManagement) {
        return modelMapper.insertSelective(maintenanceFundsManagement);
    }

    @Override
    public MaintenanceFundsManagement selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(MaintenanceFundsManagement maintenanceFundsManagement) {
        return modelMapper.updateByPrimaryKeySelective(maintenanceFundsManagement);
    }

    @Override
    public int updateByExampleSelective(MaintenanceFundsManagement maintenanceFundsManagement, MaintenanceFundsManagementExample maintenanceFundsManagementExample) {
        return modelMapper.updateByExampleSelective(maintenanceFundsManagement,maintenanceFundsManagementExample);
    }

    @Override
    public int updateByExample(MaintenanceFundsManagement maintenanceFundsManagement, MaintenanceFundsManagementExample maintenanceFundsManagementExample) {
        return modelMapper.updateByExample(maintenanceFundsManagement,maintenanceFundsManagementExample);
    }

    @Override
    public List<MaintenanceFundsManagement> selectByExample(MaintenanceFundsManagementExample maintenanceFundsManagementExample) {
        return modelMapper.selectByExample(maintenanceFundsManagementExample);
    }

    @Override
    public long countByExample(MaintenanceFundsManagementExample maintenanceFundsManagementExample) {
        return modelMapper.countByExample(maintenanceFundsManagementExample);
    }

    @Override
    public int deleteByExample(MaintenanceFundsManagementExample maintenanceFundsManagementExample) {
        return modelMapper.deleteByExample(maintenanceFundsManagementExample);
    }


    /**
     * 收款登记
     * @param model
     * @return
     */
    public int add(MaintenanceFundsManagement model){

        model.setCreatedId(ShiroUtils.getUserId());
        model.setCreatedAt(new Date());
        // 更新合同 专项维修资金收取状态 已收款
        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(model.getContractId());
        contractManage.setFundCollectionStatus(2);
        contractManageMapper.updateByPrimaryKeySelective(contractManage);
        int result = insertSelective(model);
        return result;
    }


    public PageInfo<ContractManageDto> listByModel(Tablepar tablepar, ContractManageDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ContractManageDto> list = extendMapper.listByModel(model);
        PageInfo<ContractManageDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询对应的银行
     * @param contractId
     * @return
     */
    public JSONObject getBankTaxRateByFloor(String contractId) {
        JSONObject jsonObject = new JSONObject();
        String result = "";
        // 1-房款，2-专项维修资金
        InvoiceBankExample example = new InvoiceBankExample();
        example.createCriteria().andSuitCostTypeLike("%" + "专项维修资金" + "%");
        List<InvoiceBank> bankList = invoiceBankService.selectByExample(example);
        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(contractId);
        ParkRoom parkRoom = parkRoomMapper.selectByPrimaryKey(contractManage.getHouseId());
        String discId = parkRoom.getDiscId();
        for (InvoiceBank invoiceBank : bankList) {
            if (StringUtils.isNotEmpty(invoiceBank.getSuitFloor())) {
                String[] floors = invoiceBank.getSuitFloor().split(",");
                for (String floor : floors) {
                    if (discId.equals(floor)) {
                        result = invoiceBank.getBankName() + invoiceBank.getBankBranchName() + invoiceBank.getBankNumber();
                        jsonObject.put("bankName", invoiceBank.getBankName());
                        jsonObject.put("bankBranchName", invoiceBank.getBankBranchName());
                        jsonObject.put("bankNumber", invoiceBank.getBankNumber());
                        jsonObject.put("bankFullName", result);
                        break;
                    }
                }
            }
        }
        return jsonObject;
    }

    public MaintenanceFundsManagement getById(String id){
        return modelMapper.selectByPrimaryKey(id);
    }







}
