package com.idea.paymentManagement.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkInfoExample;
import com.idea.buildManage.model.ParkRoom;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.mapper.ContractManageMapper;
import com.idea.customerManagement.model.ContractManage;
import com.idea.paymentManagement.dto.PayLogDto;
import com.idea.paymentManagement.mapper.PayLogExtendMapper;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.model.PayLog;
import com.idea.paymentManagement.model.PayLogExample;
import com.idea.util.DateUtils;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Service
public class PayLogService implements BaseService<PayLog, PayLogExample> {

    @Autowired
    private PayLogMapper modelMapper;
    @Autowired
    private PayLogExtendMapper extendMapper;
    @Autowired
    private ContractManageMapper contractManageMapper;
    @Autowired
    private FrameUserService frameUserService;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(PayLog PayLog) {
        PayLog.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(PayLog);
    }

    public int insertWithoutId(PayLog PayLog) {
        return modelMapper.insertSelective(PayLog);
    }

    @Override
    public PayLog selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(PayLog PayLog) {
        return modelMapper.updateByPrimaryKeySelective(PayLog);
    }

    @Override
    public int updateByExampleSelective(PayLog PayLog, PayLogExample PayLogExample) {
        return modelMapper.updateByExampleSelective(PayLog, PayLogExample);
    }

    @Override
    public int updateByExample(PayLog PayLog, PayLogExample PayLogExample) {
        return 0;
    }

    @Override
    public List<PayLog> selectByExample(PayLogExample PayLogExample) {
        return null;
    }

    @Override
    public long countByExample(PayLogExample PayLogExample) {
        return 0;
    }

    @Override
    public int deleteByExample(PayLogExample PayLogExample) {
        return 0;
    }


    public PageInfo<ContractManageDto> listByModel(Tablepar tablepar, ContractManageDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ContractManageDto> list = extendMapper.listByModel(model);
        PageInfo<ContractManageDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }


    /**
     * 新增
     *
     * @param model
     * @return
     */
    public int add(PayLog model) {

        model.setCreatedBy(ShiroUtils.getUserId());
        model.setCreatedAt(new Date());
        model.setStatus(0);
        int result = insertSelective(model);
        // 更新合同 预收款 收款状态
        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(model.getContractId());
        if (model.getPayType() == 1) {
            String receivedMoneyStr = extendMapper.getMoneyByContractId(model.getContractId(), model.getPayType());
            BigDecimal totalPrice = contractManage.getTotalPrice();
            BigDecimal receivedMoney = new BigDecimal(receivedMoneyStr);
            if (receivedMoney.doubleValue() < totalPrice.doubleValue()) {
                // 部分收款
                contractManage.setCollectionStatus(1);
            } else {
                // 全部收款
                contractManage.setCollectionStatus(2);
            }
        }
        // 更新合同 资金 收款状态
        if (model.getPayType() == 2) {
            // 已收款
            contractManage.setFundCollectionStatus(2);
        }
        contractManageMapper.updateByPrimaryKeySelective(contractManage);
        return result;
    }


    /**
     * 收款信息
     *
     * @param model
     * @param dt
     * @return
     */
    public List<PayLog> payListAll(PayLog model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    private PayLogExample getCondition(PayLog model, DateTrans dt) {
        PayLogExample ex = new PayLogExample();
        ex.setOrderByClause("created_at asc");
        PayLogExample.Criteria criteria = ex.createCriteria();
        if (StringUtils.isNotEmpty(model.getContractId())) {
            criteria.andContractIdEqualTo(model.getContractId());
        }
        return ex;
    }

    /**
     * 查询合同部分的信息
     *
     * @param contractId
     * @return
     */
    public PayLogDto getContractPart(String contractId) {

        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(contractId);
        PayLogDto payLogDto = new PayLogDto();
        payLogDto.setContractNumber(contractManage.getContractNumber());
        payLogDto.setBuyerName(contractManage.getBuyerName());
        payLogDto.setHouseName(contractManage.getHouseName());
        payLogDto.setPaymentMethod(contractManage.getPaymentMethod());
        payLogDto.setDeadLine(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, contractManage.getDeadline()));
        payLogDto.setReceivableMoney(contractManage.getTotalPrice());
        // 查询已收房款
        String receivedMoneyStr = extendMapper.getMoneyByContractId(contractId, 1);
        BigDecimal receivedMoney = new BigDecimal(receivedMoneyStr);
        // 欠款
        BigDecimal arrears = contractManage.getTotalPrice().subtract(receivedMoney);
        payLogDto.setReceivedMoney(receivedMoney);
        payLogDto.setArrears(arrears);
        return payLogDto;
    }

    public PayLog getByContractId(String contractId, Integer payType) {

        PayLogExample payLogExample = new PayLogExample();
        payLogExample.createCriteria().andContractIdEqualTo(contractId).andPayTypeEqualTo(payType);
        List<PayLog> payLogs = modelMapper.selectByExample(payLogExample);
        return payLogs.get(0);

    }

    public PayLogDto getById(String id) {
        PayLog payLog = selectByPrimaryKey(id);
        String username = "";
        if (StringUtils.isNotEmpty(payLog.getCreatedBy())) {
            FrameUser frameUser = frameUserService.selectByPrimaryKey(payLog.getCreatedBy());
            username = frameUser == null ? "" : frameUser.getTruename();
        }
        PayLogDto result = new PayLogDto();
        result.setCreatedName(username);
        BeanUtil.copyProperties(payLog, result);
        return result;
    }


}
