package com.idea.paymentManagement.service;

import cn.hutool.core.util.IdUtil;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.mapper.RefundDetailMapper;
import com.idea.paymentManagement.model.PayLog;
import com.idea.paymentManagement.model.PayLogExample;
import com.idea.paymentManagement.model.RefundDetail;
import com.idea.paymentManagement.model.RefundDetailExample;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class RefundDetailService implements BaseService<RefundDetail, RefundDetailExample> {

    @Autowired
    private RefundDetailMapper modelMapper;
    @Autowired
    private PayLogMapper payLogMapper;



    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(RefundDetail refundDetail) {
        refundDetail.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(refundDetail);
    }

    public int insertWithoutId(RefundDetail refundDetail) {
        return modelMapper.insertSelective(refundDetail);
    }

    @Override
    public RefundDetail selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(RefundDetail refundDetail) {
        return modelMapper.updateByPrimaryKeySelective(refundDetail);
    }

    @Override
    public int updateByExampleSelective(RefundDetail refundDetail, RefundDetailExample refundDetailExample) {
        return modelMapper.updateByExampleSelective(refundDetail,refundDetailExample);
    }

    @Override
    public int updateByExample(RefundDetail refundDetail, RefundDetailExample refundDetailExample) {
        return modelMapper.updateByExample(refundDetail,refundDetailExample);
    }

    @Override
    public List<RefundDetail> selectByExample(RefundDetailExample refundDetailExample) {
        return modelMapper.selectByExample(refundDetailExample);
    }

    @Override
    public long countByExample(RefundDetailExample refundDetailExample) {
        return modelMapper.countByExample(refundDetailExample);
    }

    @Override
    public int deleteByExample(RefundDetailExample refundDetailExample) {
        return modelMapper.deleteByExample(refundDetailExample);
    }


    /**
     * 根据轮号客户id 查询已收 部分收款 完全收款
     * @param customerManagementId
     * @return
     */
    public List<RefundDetail> getByCustomerManagementId(String customerManagementId){

        PayLogExample payLogExample = new PayLogExample();
        payLogExample.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<PayLog> payLogs = payLogMapper.selectByExample(payLogExample);
        List<RefundDetail> refundDetails = new ArrayList<>();
        for(PayLog payLog : payLogs){
            RefundDetail refundDetail = new RefundDetail();
            refundDetail.setContentType(payLog.getContentType());
            refundDetail.setReceivedAmount(payLog.getPayMoney());
            refundDetails.add(refundDetail);
        }
        return refundDetails;
    }

    public List<RefundDetail> listAll(RefundDetail model,DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }


    private RefundDetailExample getCondition(RefundDetail model,DateTrans dt) {
        RefundDetailExample ex = new RefundDetailExample();
        ex.setOrderByClause("created_at");
        RefundDetailExample.Criteria criteria = ex.createCriteria();
        if (StringUtils.isNotEmpty(model.getRefundManageId())) {
            criteria.andRefundManageIdEqualTo(model.getRefundManageId());
        }
        return ex;
    }

}
