package com.idea.paymentManagement.service;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.Buyer;
import com.idea.customerManagement.model.BuyerExample;
import com.idea.customerManagement.model.RoomSelectionInfo;
import com.idea.customerManagement.model.RoomSelectionInfoExample;
import com.idea.customerManagement.service.BuyerService;
import com.idea.paymentManagement.dto.RefundManageDto;
import com.idea.paymentManagement.mapper.RefundManageExtendMapper;
import com.idea.paymentManagement.mapper.RefundManageMapper;
import com.idea.paymentManagement.model.RefundDetail;
import com.idea.paymentManagement.model.RefundDetailExample;
import com.idea.paymentManagement.model.RefundManage;
import com.idea.paymentManagement.model.RefundManageExample;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class RefundManageService implements BaseService<RefundManage, RefundManageExample> {

    @Autowired
    private RefundManageMapper modelMapper;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private RoomSelectionInfoMapper roomSelectionInfoMapper;
    @Autowired
    private ParkRoomExtendMapper roomExtendMapper;
    @Autowired
    private RefundDetailService refundDetailService;
    @Autowired
    private RefundManageExtendMapper extendMapper;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(RefundManage refundManage) {
        refundManage.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(refundManage);
    }

    public int insertWithoutId(RefundManage refundManage) {
        return modelMapper.insertSelective(refundManage);
    }

    @Override
    public RefundManage selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(RefundManage refundManage) {
        return modelMapper.updateByPrimaryKeySelective(refundManage);
    }

    @Override
    public int updateByExampleSelective(RefundManage refundManage, RefundManageExample refundManageExample) {
        return modelMapper.updateByExampleSelective(refundManage,refundManageExample);
    }

    @Override
    public int updateByExample(RefundManage refundManage, RefundManageExample refundManageExample) {
        return modelMapper.updateByExample(refundManage,refundManageExample);
    }

    @Override
    public List<RefundManage> selectByExample(RefundManageExample refundManageExample) {
        return modelMapper.selectByExample(refundManageExample);
    }

    @Override
    public long countByExample(RefundManageExample refundManageExample) {
        return modelMapper.countByExample(refundManageExample);
    }

    @Override
    public int deleteByExample(RefundManageExample refundManageExample) {
        return modelMapper.deleteByExample(refundManageExample);
    }


    /**
     * 根据身份证号带出关联房屋
     * @param identityCard
     * @return
     */
    public List<ParkRoomResponse> getHouseListByIdCard(String identityCard){

        // 根据身份证查询 买方信息
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.createCriteria().andIdentityCardEqualTo(identityCard);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        List<String> customerManagementIds = buyers.stream().map(Buyer::getCustomerManagementId).collect(Collectors.toList());
        // 根据客户管理信息 查询选房信息
        RoomSelectionInfoExample roomSelectionInfoExample = new RoomSelectionInfoExample();
        roomSelectionInfoExample.createCriteria().andCustomerManagementIdIn(customerManagementIds);
        List<RoomSelectionInfo> selectionList = roomSelectionInfoMapper.selectByExample(roomSelectionInfoExample);
        List<ParkRoomResponse> result = new ArrayList<>();
        for(RoomSelectionInfo info : selectionList){
            ParkRoomResponse response = roomExtendMapper.getFullName(info.getHouseId());
            response.setCustomerManagementId(info.getCustomerManagementId());
            result.add(response);
        }
        return result;
    }


    /**
     * 新增收款
     * @param model
     * @return
     */
    public int add(RefundManageDto model){

        String id = IdUtil.simpleUUID();
        String userId = ShiroUtils.getUserId();
        Date date = new Date();
        List<RefundDetail> details = JSONObject.parseArray(model.getRefundDetailServiceStr(), RefundDetail.class);
        for (RefundDetail detail : details){
            detail.setRefundManageId(id);
            detail.setCreatedId(userId);
            detail.setCreatedAt(date);
            refundDetailService.insertSelective(detail);
        }
        model.setId(id);
        model.setCreatedId(userId);
        model.setCreatedAt(date);
        int result = insertWithoutId(model);
        return result;
    }

    /**
     * 编辑收款
     * @param model
     * @return
     */
    public int edit(RefundManageDto model){

        String id = model.getId();
        String userId = ShiroUtils.getUserId();
        Date date = new Date();
        // 先删除
        RefundDetailExample refundDetailExample = new RefundDetailExample();
        refundDetailExample.createCriteria().andRefundManageIdEqualTo(model.getId());
        List<RefundDetail> oldDetails = refundDetailService.selectByExample(refundDetailExample);
        for(RefundDetail detail : oldDetails){
            refundDetailService.deleteByPrimaryKey(detail.getId());
        }
        // 再新增
        List<RefundDetail> details = JSONObject.parseArray(model.getRefundDetailServiceStr(), RefundDetail.class);
        for (RefundDetail detail : details){
            detail.setRefundManageId(id);
            detail.setCreatedId(userId);
            detail.setCreatedAt(date);
            refundDetailService.insertSelective(detail);
        }
        model.setId(id);
        model.setUpdatedId(userId);
        model.setUpdatedAt(date);
        int result = updateByPrimaryKeySelective(model);
        return result;
    }


    public PageInfo<RefundManageDto> listByModel(Tablepar tablepar, RefundManageDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<RefundManageDto> list = extendMapper.listByModel(model);
        PageInfo<RefundManageDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }





}
